'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _readJson = require('./utils/read-json');

var _readJson2 = _interopRequireDefault(_readJson);

var _writeJson = require('./utils/write-json');

var _writeJson2 = _interopRequireDefault(_writeJson);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _class = function () {
  function _class(config) {
    _classCallCheck(this, _class);

    this.config = config;

    this.storageLocation = _path2.default.join(this.config.workDir, this.config.storage.directory);

    try {
      if (!_fs2.default.existsSync(this.storageLocation)) {
        _mkdirp2.default.sync(this.storageLocation);
      }
    } catch (err) {
      console.log("Could not create storage directory, ap-npm might malfunction\n", err.toString());
    }
  }

  _createClass(_class, [{
    key: 'removePackage',
    value: function removePackage(packageName) {
      var _this = this;

      return new Promise(function (resolve, reject) {
        var packageLocation = _path2.default.join(_this.storageLocation, packageName);

        if (!_fs2.default.existsSync(packageLocation + '/package.json')) {
          reject("Invalid request, aborting");
        }

        // location is valid
        (0, _rimraf2.default)(packageLocation, function () {
          resolve(true);
        });
      }).catch(function (err) {
        throw new Error(err);
      });
    }
  }, {
    key: 'removePackageVersion',
    value: function removePackageVersion(packageName, packageVersion) {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        var packageLocation = _this2.storageLocation + '/' + packageName;
        var tarballLocation = packageLocation + '/' + packageName + '-' + packageVersion + '.tgz';

        _fs2.default.exists(packageLocation + '/package.json', function (exists) {
          if (!exists) {
            reject("Invalid request, aborting");
          }

          // location is valid
          _fs2.default.unlink(tarballLocation, function () {
            var packageJson = _this2.getPackageJson(packageName);

            delete packageJson.versions[packageVersion];

            // If this was the last version of the package, we can remove it completely
            if (packageJson.versions.size === 0) {
              _this2.removePackage(packageName);
              return true;
            }

            if (packageJson['dist-tags']['latest'] === packageVersion) {
              // need to update dist-tags
              var highestVersion = '0.0.1';
              for (var key in packageJson.versions) {
                if (_semver2.default.satisfies(key, '>' + highestVersion)) {
                  highestVersion = key;
                }
              }
              packageJson['dist-tags']['latest'] = highestVersion;
            }
            _this2.updatePackageJson(packageName, packageJson).then(function (result) {
              resolve(result);
            });
          });
        });
      }).catch(function (err) {
        throw new Error(err);
      });
    }

    // This is used for packages that don't exist in the storage yet

  }, {
    key: 'writeNewPackage',
    value: function writeNewPackage(packageData) {
      var _this3 = this;

      return new Promise(function (resolve) {
        var fileName = void 0;
        var packageName = packageData._packageName;
        var packageScope = packageData._scope;
        var attachmentName = void 0;
        var folderPath = void 0;
        var packageJsonPath = void 0;
        var filePath = void 0;
        for (var key in packageData._attachments) {
          attachmentName = key;
        }

        if (packageScope) {
          folderPath = _path2.default.join(_this3.storageLocation, packageScope, packageName);
          filePath = _path2.default.join(folderPath, attachmentName.substr(packageScope.length + 1));
        } else {
          folderPath = _path2.default.join(_this3.storageLocation, packageName);
          filePath = _path2.default.join(folderPath, attachmentName);
        }

        _mkdirp2.default.sync(folderPath);
        packageJsonPath = _path2.default.join(folderPath, 'package.json');

        _fs2.default.writeFileSync(filePath, Buffer.from(packageData._attachments[attachmentName].data, 'base64'), { 'mode': '0777' });
        var packageJSON = packageData;
        delete packageJSON._attachments;

        (0, _writeJson2.default)(packageJsonPath, packageJSON).then(function (result) {
          console.log("Wrote new package to filesystem:", {
            "filePath": filePath,
            "packageJSON": packageJsonPath
          });
          resolve(result);
        });
      }).catch(function (err) {
        throw new Error(err);
      });
    }

    // This is used for packages that have been published before -> adding a new version

  }, {
    key: 'writePackage',
    value: function writePackage(packageData) {
      var _this4 = this;

      return new Promise(function (resolve) {
        var attachmentName = void 0;
        var packageName = packageData._packageName;
        var packageScope = packageData._scope;
        var packageInfoLocation = void 0;
        var folderPath = void 0;

        if (packageScope) {
          packageInfoLocation = _path2.default.join(_this4.storageLocation, packageScope, packageName, 'package.json');
          folderPath = _path2.default.join(_this4.storageLocation, packageScope, packageName);
        } else {
          packageInfoLocation = _path2.default.join(_this4.storageLocation, packageName, 'package.json');
          folderPath = _path2.default.join(_this4.storageLocation, packageName);
        }

        _mkdirp2.default.sync(folderPath);
        for (var key in packageData._attachments) {
          attachmentName = key;
        }

        var newVersion = void 0;
        for (var _key in packageData.versions) {
          newVersion = _key;
        }

        var filePath = folderPath + '/' + packageName + '-' + newVersion + '.tgz';

        (0, _readJson2.default)(packageInfoLocation).then(function (packageJSON) {
          packageJSON.versions[newVersion] = packageData.versions[newVersion];

          var distTags = packageJSON['dist-tags'];
          var newDistTags = packageData['dist-tags'];

          // Merge dist-tags, we need to preserve old dist-tags
          for (var _key2 in newDistTags) {
            distTags[_key2] = newDistTags[_key2];
          }

          packageJSON['dist-tags'] = distTags;

          console.log(attachmentName);
          _fs2.default.writeFile(filePath, Buffer.from(packageData._attachments[attachmentName].data, 'base64'), { 'mode': '0777' }, function () {
            (0, _writeJson2.default)(packageInfoLocation, packageJSON).then(function (result) {
              console.log("Wrote package to filesystem:", {
                "filePath": filePath,
                "packageJSON": packageInfoLocation
              });
              resolve(result);
            });
          });
        });
      }).catch(function (err) {
        throw new Error(err);
      });
    }
  }, {
    key: 'getPackage',
    value: function getPackage(request) {
      var _this5 = this;

      var packageName = request.name;
      var packageScope = request.scope;
      var fileName = request.file;

      return new Promise(function (resolve, reject) {
        var fileLocation = void 0;

        if (packageScope) {
          fileLocation = _path2.default.join(_this5.storageLocation, packageScope, packageName, fileName);
        } else {
          fileLocation = _path2.default.join(_this5.storageLocation, packageName, fileName);
        }

        _fs2.default.readFile(fileLocation, function (err, file) {
          if (err) {
            console.log("Err: file does not exist " + fileName);
            reject(err);
          }
          resolve(file);
        });
      });
    }
  }, {
    key: 'getPackageData',
    value: function getPackageData(request) {
      var _this6 = this;

      return new Promise(function (resolve, reject) {
        var packageName = request.name;
        var packageScope = request.scope;

        var jsonPath = void 0;
        if (packageScope) {
          jsonPath = _path2.default.join(_this6.storageLocation, packageScope, packageName, 'package.json');
        } else {
          jsonPath = _path2.default.join(_this6.storageLocation, packageName, 'package.json');
        }

        if (_fs2.default.existsSync(jsonPath)) {
          (0, _readJson2.default)(jsonPath).then(function (data) {
            resolve(data);
          });
        } else {
          reject("Could not get package.json");
        }
      });
    }

    // *** STORAGE VALIDATION ***
    // Checks if our storage has an entry for this packageName

  }, {
    key: 'isPackageAvailable',
    value: function isPackageAvailable(packageName) {
      var _this7 = this;

      var packageScope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      return new Promise(function (resolve) {
        var packagePath = void 0;
        if (packageScope) {
          packagePath = _path2.default.join(_this7.storageLocation, packageScope, packageName, 'package.json');
        } else {
          packagePath = _path2.default.join(_this7.storageLocation, packageName, 'package.json');
        }

        if (_fs2.default.existsSync(packagePath)) {
          resolve(true);
        }
        resolve(false);
      }).catch(function (err) {
        console.log("Package not available: " + packageName);
        return false;
      });
    }

    // Checks if a certain version of a package actually exists

  }, {
    key: 'isVersionAvailable',
    value: function isVersionAvailable(request, packageVersion) {
      var _this8 = this;

      var packageName = request._packageName;
      var packageScope = request._scope;
      var scopedName = request._scopedName;

      return new Promise(function (resolve) {
        var packageInfoLocation = void 0;
        if (packageScope) {
          packageInfoLocation = _path2.default.join(_this8.storageLocation, packageScope, packageName, 'package.json');
        } else {
          packageInfoLocation = _path2.default.join(_this8.storageLocation, packageName, 'package.json');
        }

        (0, _readJson2.default)(packageInfoLocation).then(function (packageJSON) {
          var versionExists = false;
          var fileName = void 0;

          for (var version in packageJSON.versions) {
            if (version === packageVersion) {
              versionExists = true;
            }
          }

          if (packageScope) {
            fileName = packageName.substr(packageScope.length + 1);
          } else {
            fileName = packageName;
          }

          var fileExists = void 0;
          if (packageScope) {
            fileExists = _fs2.default.existsSync(_path2.default.join(_this8.storageLocation, packageScope, packageName, packageName + '-' + packageVersion + '.tgz'));
          } else {
            fileExists = _fs2.default.existsSync(_path2.default.join(_this8.storageLocation, packageName, fileName + '-' + packageVersion + '.tgz'));
          }

          // Both have to be true for the version requested to be available
          resolve(versionExists && fileExists);
        });
      }).catch(function (err) {
        console.log(err);
        return false;
      });
    }
  }, {
    key: 'getPackageJson',
    value: function getPackageJson(packageName) {
      var _this9 = this;

      var packageScope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      return new Promise(function (resolve) {

        var packageInfoLocation = void 0;
        if (packageScope) {
          packageInfoLocation = _path2.default.join(_this9.storageLocation, packageScope, packageName, 'package.json');
        } else {
          packageInfoLocation = _path2.default.join(_this9.storageLocation, packageName, 'package.json');
        }

        (0, _readJson2.default)(packageInfoLocation).then(function (data) {
          return resolve(data);
        });
      });
    }

    /* Dist-tag functions*/

  }, {
    key: 'updatePackageJson',
    value: function updatePackageJson(packageData) {
      var _this10 = this;

      var packageName = packageData._packageName;
      var packageScope = packageData._scope;
      var scopedName = packageData._scopedName;
      return new Promise(function (resolve) {
        var packageInfoLocation = void 0;
        if (packageScope) {
          packageInfoLocation = _path2.default.join(_this10.storageLocation, packageScope, packageName, 'package.json');
        } else {
          packageInfoLocation = _path2.default.join(_this10.storageLocation, packageName, 'package.json');
        }

        (0, _writeJson2.default)(packageInfoLocation, packageJson).then(function (result) {
          return resolve(result);
        });
      });
    }
  }]);

  return _class;
}();

exports.default = _class;